/* xml++.h
 * libxml++ and this file are copyright (C) 2000 by Ari Johnson, and
 * are covered by the GNU Lesser General Public License, which should be
 * included with libxml++ as the file COPYING.
 */

#ifndef __LIBXMLPP_PARSER_H
#define __LIBXMLPP_PARSER_H

#include <libxml++/nodes/element.h>
#include <libxml++/exceptions/validity_error.h>
#include <libxml++/exceptions/internal_error.h>

#include <iostream>

namespace xmlpp {

/** XML parser.
 *
 */
class Parser
{
public:
  Parser();
  virtual ~Parser();

  virtual void set_validate(bool val = true);
  virtual bool get_validate() const;

  virtual void parse_file(const std::string& filename) throw(exception) = 0;
  virtual void parse_memory(const std::string& contents) throw(exception) = 0;
  virtual void parse_stream(std::istream& in) throw(exception) = 0;

protected:
  virtual void initialize_context();
  virtual void release_underlying();

  virtual void on_validity_error(const std::string& message);
  virtual void on_validity_warning(const std::string& message);

  virtual void handleException(const exception& e);
  virtual void check_for_exception();
  virtual void check_for_validity_messages();
  
  static void _callback_validity_error(void* ctx, const char* msg, ...);
  static void _callback_validity_warning(void* ctx, const char* msg, ...);
  
  enum enumConstants
  {
    CONSTANT_KeepBlanksSetting = 0 // =FALSE - Don't use libxml1 compatability. This is the default anyway.
  };
  
  xmlParserCtxtPtr _context;
  exception* _exception;
  std::string _validate_error, _validate_warning; //Built gradually - used in an exception at the end of parsing.

  bool _validate;
};

} // namespace xmlpp

#endif //__LIBXMLPP_PARSER_H



